@@ -1,4 +1,5 @@
 mod completions;
+mod parallel;
 mod utils;
 
 use anyhow::{anyhow, bail, Context, Result};
@@ -6,6 +7,7 @@ use argc::{
     utils::{escape_shell_words, get_current_dir, get_shell_path, termwidth},
     Shell,
 };
+use base64::{engine::general_purpose, Engine as _};
 use std::{
     collections::HashMap,
     env, fs, process,
@@ -15,6 +17,9 @@ use std::{
     },
 };
 use utils::*;
+use which::which;
+
+const COMPLETION_SCRIPT: &str = include_str!("completions/completion.sh");
 
 fn main() {
     match run() {
@@ -43,12 +48,31 @@ fn run() -> Result<i32> {
         match argc_cmd {
             "--argc-eval" => {
                 let (source, cmd_args) = parse_script_args(&args[2..])?;
-                let values = argc::eval(&source, &cmd_args, Some(&args[2]), termwidth())?;
-                let export_pwd = match env::var("ARGC_PWD").ok().or_else(get_current_dir) {
-                    Some(v) => format!("export ARGC_PWD={v}\n"),
-                    None => String::new(),
-                };
-                println!("{export_pwd}{}", argc::ArgcValue::to_shell(values))
+                if cmd_args
+                    .get(1)
+                    .map(|v| v == parallel::PARALLEL_MODE)
+                    .unwrap_or_default()
+                {
+                    let cmd_args_len = cmd_args.len();
+                    if cmd_args_len < 3 {
+                        bail!("No parallel command")
+                    }
+                    let mut code = retrive_argc_variables().unwrap_or_default();
+                    let mut cmds = vec![cmd_args[2].to_string()];
+                    cmds.extend(cmd_args[3..].iter().map(|v| escape_shell_words(v)));
+                    code.push_str(&cmds.join(" "));
+                    println!("{code}")
+                } else {
+                    let values = argc::eval(&source, &cmd_args, Some(&args[2]), termwidth())
+                        .map_err(|err| anyhow!("error: {err}"))?;
+                    let export_pwd = match env::var("ARGC_PWD").ok().or_else(get_current_dir) {
+                        Some(v) => format!("export ARGC_PWD={v}\n"),
+                        None => String::new(),
+                    };
+                    let code = argc::ArgcValue::to_shell(&values);
+                    let export_vars = export_argc_variables(&code);
+                    println!("{export_pwd}{export_vars}{code}")
+                }
             }
             "--argc-create" => {
                 if let Some((_, script_file)) = get_script_path(false) {
@@ -66,15 +90,7 @@ fn run() -> Result<i32> {
                 println!("{}", serde_json::to_string_pretty(&json)?);
             }
             "--argc-compgen" => {
-                let shell: Shell = match args.get(2) {
-                    Some(v) => v.parse()?,
-                    None => bail!("Usage: argc --argc-compgen <SHELL> <SCRIPT> <ARGS...>"),
-                };
-                let (source, cmd_args) = parse_script_args(&args[3..])?;
-                let output = argc::compgen(shell, &args[3], &source, &cmd_args[1..])?;
-                if !output.is_empty() {
-                    println!("{output}");
-                }
+                run_compgen(args.to_vec());
             }
             "--argc-completions" => {
                 let shell: Shell = match args.get(2) {
@@ -84,6 +100,18 @@ fn run() -> Result<i32> {
                 let script = crate::completions::generate(shell, &args[3..])?;
                 println!("{}", script);
             }
+            "--argc-parallel" => {
+                if args.len() <= 3 {
+                    bail!("Usage: argc --argc-parallel <SCRIPT> <ARGS...>");
+                }
+                let shell = get_shell_path().ok_or_else(|| anyhow!("Shell not found"))?;
+                let (source, cmd_args) = parse_script_args(&args[2..])?;
+                if !source.contains("--argc-eval") {
+                    bail!("Parallel only available for argc based scripts.")
+                }
+                let script_file = args[2].clone();
+                parallel::parallel(&shell, &script_file, &cmd_args[1..])?;
+            }
             "--argc-script-path" => {
                 let (_, script_file) =
                     get_script_path(true).ok_or_else(|| anyhow!("Argcfile not found."))?;
@@ -96,7 +124,7 @@ fn run() -> Result<i32> {
                 println!("{}", get_argc_version())
             }
             _ => {
-                bail!("Invalid option `{argc_cmd}`")
+                bail!("Unknown option `{argc_cmd}`")
             }
         }
         Ok(0)
@@ -127,6 +155,67 @@ fn run() -> Result<i32> {
     }
 }
 
+fn run_compgen(mut args: Vec<String>) -> Option<()> {
+    if args.len() < 6 {
+        return None;
+    };
+    let shell: Shell = args.get(2).and_then(|v| v.parse().ok())?;
+    if args[3].is_empty() {
+        if args[4] == "argc" {
+            if let Some((_, script_file_)) = get_script_path(true) {
+                args[3] = script_file_.to_string_lossy().to_string();
+            }
+        } else if let Ok(script_file_) = which(&args[4]) {
+            args[3] = script_file_.to_string_lossy().to_string();
+        } else {
+            return None;
+        }
+    }
+    let no_color = if let Ok(v) = std::env::var("NO_COLOR") {
+        v == "true" || v == "1"
+    } else {
+        false
+    };
+    let output = if args.len() >= 6
+        && &args[4] == "argc"
+        && (args[3].is_empty() || args[5].starts_with("--argc"))
+    {
+        let cmd_args = &args[4..];
+        argc::compgen(shell, "", COMPLETION_SCRIPT, cmd_args, no_color).ok()?
+    } else {
+        let (source, cmd_args) = parse_script_args(&args[3..]).ok()?;
+        argc::compgen(shell, &args[3], &source, &cmd_args[1..], no_color).ok()?
+    };
+    if !output.is_empty() {
+        println!("{output}");
+    }
+    Some(())
+}
+
+fn export_argc_variables(code: &str) -> String {
+    let mut value = code
+        .split('\n')
+        .filter(|line| line.starts_with(argc::VARIABLE_PREFIX))
+        .map(|v| v.to_string())
+        .collect::<Vec<String>>()
+        .join(";");
+    if value.is_empty() {
+        return value;
+    } else {
+        value.push(';');
+    }
+    format!(
+        "export ARGC_VARS={}\n",
+        general_purpose::STANDARD.encode(value)
+    )
+}
+
+fn retrive_argc_variables() -> Option<String> {
+    let value = env::var("ARGC_VARS").ok()?;
+    let value = general_purpose::STANDARD.decode(value).ok()?;
+    String::from_utf8(value).ok()
+}
+
 fn get_argc_help() -> String {
     let about = concat!(
         env!("CARGO_PKG_DESCRIPTION"),
@@ -142,6 +231,7 @@ USAGE:
     argc --argc-completions <SHELL> [CMDS...]       Generate completion scripts for bash,elvish,fish,nushell,powershell,xsh,zsh
     argc --argc-compgen <SHELL> <SCRIPT> <ARGS...>  Generate dynamic completion word
     argc --argc-export <SCRIPT>                     Export command line definitions as json
+    argc --argc-parallel <SCRIPT> <ARGS...>         Execute argc functions in parallel
     argc --argc-script-path                         Print current argcfile path
     argc --argc-help                                Print help information
     argc --argc-version                             Print version information
