@@ -8,6 +8,7 @@ use crate::argc_value::ArgcValue;
 use crate::matcher::Matcher;
 use crate::param::{FlagOptionParam, PositionalParam};
 use crate::parser::{parse, Event, EventData, EventScope, Position};
+use crate::utils::INTERNAL_MODE;
 use crate::Result;
 
 use anyhow::{bail, Context};
@@ -64,16 +65,16 @@ impl Command {
         if args.is_empty() {
             bail!("Invalid args");
         }
-        if args.len() >= 2 && self.root.borrow().exist_param_fn(args[1].as_str()) {
+        if args.len() >= 3 && args[1] == INTERNAL_MODE {
             let fallback_args = vec!["prog".to_string()];
-            let new_args = if args.len() == 2 {
+            let new_args = if args.len() == 3 {
                 &fallback_args
             } else {
-                &args[2..]
+                &args[3..]
             };
             let matcher = Matcher::new(self, new_args);
             let mut arg_values = matcher.to_arg_values_for_choice_fn();
-            arg_values.push(ArgcValue::ParamFn(args[1].clone()));
+            arg_values.push(ArgcValue::ParamFn(args[2].clone()));
             return Ok(arg_values);
         }
         let mut matcher = Matcher::new(self, args);
@@ -132,7 +133,10 @@ impl Command {
                 }
                 EventData::Cmd(value) => {
                     if root_data.borrow().scope == EventScope::CmdStart {
-                        bail!("@cmd(line {}) miss function?", root_data.borrow().cmd_pos)
+                        bail!(
+                            "@cmd(line {}) missing function?",
+                            root_data.borrow().cmd_pos
+                        )
                     }
                     root_data.borrow_mut().cmd_pos = position;
                     root_data.borrow_mut().scope = EventScope::CmdStart;
@@ -170,7 +174,7 @@ impl Command {
                 EventData::Func(name) => {
                     if let Some(pos) = root_data.borrow_mut().cmd_fns.get(&name) {
                         bail!(
-                            "{}(line {}) is conflicted with cmd or alias at line {}",
+                            "{}(line {}) conflicts with cmd or alias at line {}",
                             name,
                             position,
                             pos
@@ -194,10 +198,10 @@ impl Command {
                             for name in &cmd.aliases {
                                 if let Some(pos) = root_data.borrow().cmd_fns.get(name) {
                                     bail!(
-										"@alias(line {}) is conflicted with cmd or alias at line {}",
-										cmd.alias_pos,
-										pos
-									);
+                                        "@alias(line {}) conflicts with cmd or alias at line {}",
+                                        cmd.alias_pos,
+                                        pos
+                                    );
                                 }
                                 root_data
                                     .borrow_mut()
@@ -217,7 +221,7 @@ impl Command {
                                     for name in &cmd.aliases {
                                         if let Some(pos) = parent_cmd.subcommand_fns.get(name) {
                                             bail!(
-												"@alias(line {}) is conflicted with cmd or alias at line {}",
+												"@alias(line {}) conflicts with cmd or alias at line {}",
 												cmd.alias_pos,
 												pos
 											);
@@ -237,7 +241,7 @@ impl Command {
                     root_data.borrow_mut().scope = EventScope::FnEnd;
                 }
                 EventData::Unknown(name) => {
-                    bail!("@{}(line {}) is unknown", name, position);
+                    bail!("@{}(line {}) is unknown tag", name, position);
                 }
             }
         }
@@ -508,7 +512,7 @@ impl Command {
     fn get_cmd<'a>(cmd: &'a mut Self, tag_name: &str, position: usize) -> Result<&'a mut Self> {
         if cmd.root.borrow().scope == EventScope::FnEnd {
             bail!(
-                "{}(line {}) is unexpected, maybe miss @cmd?",
+                "{}(line {}) shouldn't be here, @cmd is missing?",
                 tag_name,
                 position
             )
