@@ -1,7 +1,12 @@
-use std::collections::{HashMap, HashSet};
+use std::{
+    collections::{HashMap, HashSet},
+    env,
+    path::MAIN_SEPARATOR,
+};
 
 use crate::{
     command::Command,
+    compgen::CompKind,
     param::{FlagOptionParam, PositionalParam},
     utils::run_param_fns,
     ArgcValue,
@@ -17,12 +22,13 @@ pub(crate) struct Matcher<'a, 'b> {
     args: &'b [String],
     flag_option_args: Vec<Vec<FlagOptionArg<'a, 'b>>>,
     positional_args: Vec<&'b str>,
-    dashdash: Vec<usize>,
+    dashes: Vec<usize>,
     arg_comp: ArgComp,
     choices_fns: HashSet<&'a str>,
     choices_values: HashMap<&'a str, Vec<String>>,
     script_path: Option<String>,
     term_width: Option<usize>,
+    is_last_arg_option_assign: bool,
 }
 
 type FlagOptionArg<'a, 'b> = (&'b str, Vec<&'b str>, Option<&'a str>);
@@ -63,8 +69,9 @@ impl<'a, 'b> Matcher<'a, 'b> {
         let mut arg_index = 1;
         let mut flag_option_args = vec![vec![]];
         let mut positional_args = vec![];
-        let mut dashdash = vec![];
-        let mut terminated = false;
+        let mut dashes = vec![];
+        let mut is_rest_args_positional = false;
+        let mut is_last_arg_option_assign = false;
         let mut arg_comp = ArgComp::Any;
         let mut choices_fns = HashSet::new();
         let args_len = args.len();
@@ -79,12 +86,20 @@ impl<'a, 'b> Matcher<'a, 'b> {
             let cmd = cmds[cmd_level].1;
             let arg = args[arg_index].as_str();
             if arg == "--" {
-                dashdash.push(positional_args.len());
-            } else if terminated
-                || !dashdash.is_empty()
+                dashes.push(positional_args.len());
+                if is_rest_args_positional {
+                    add_positional_arg(
+                        &mut positional_args,
+                        arg,
+                        &mut is_rest_args_positional,
+                        cmd,
+                    );
+                }
+            } else if is_rest_args_positional
+                || !dashes.is_empty()
                 || (cmd.no_flags_options_subcommands() && !KNOWN_OPTIONS.contains(&arg))
             {
-                add_positional_arg(&mut positional_args, arg, &mut terminated, cmd);
+                add_positional_arg(&mut positional_args, arg, &mut is_rest_args_positional, cmd);
             } else if arg.starts_with('-') {
                 if let Some((k, v)) = arg.split_once('=') {
                     let param = cmd.find_flag_option(k);
@@ -92,6 +107,7 @@ impl<'a, 'b> Matcher<'a, 'b> {
                         if let Some(param) = param {
                             arg_comp = ArgComp::OptionValue(param.name.clone(), 0)
                         }
+                        is_last_arg_option_assign = true;
                     }
                     if let Some((choices_fn, validate)) = param.and_then(|v| v.choices_fn.as_ref())
                     {
@@ -142,12 +158,12 @@ impl<'a, 'b> Matcher<'a, 'b> {
                 ));
                 flag_option_args.push(vec![]);
             } else {
-                add_positional_arg(&mut positional_args, arg, &mut terminated, cmd);
+                add_positional_arg(&mut positional_args, arg, &mut is_rest_args_positional, cmd);
             }
             arg_index += 1;
         }
 
-        if terminated {
+        if is_rest_args_positional {
             arg_comp = ArgComp::CommandOrPositional;
         }
 
@@ -165,19 +181,23 @@ impl<'a, 'b> Matcher<'a, 'b> {
             args,
             flag_option_args,
             positional_args,
-            dashdash,
+            dashes,
             arg_comp,
             choices_fns,
             choices_values: HashMap::new(),
             script_path: None,
             term_width: None,
+            is_last_arg_option_assign,
         }
     }
 
     pub(crate) fn set_script_path(&mut self, script_path: &str) {
         self.script_path = Some(script_path.to_string());
         let fns: Vec<&str> = self.choices_fns.iter().copied().collect();
-        if let Some(outputs) = run_param_fns(script_path, &fns, self.args, HashMap::new()) {
+        let mut envs = HashMap::new();
+        envs.insert("ARGC_OS".into(), env::consts::OS.to_string());
+        envs.insert("ARGC_PATH_SEP".into(), MAIN_SEPARATOR.into());
+        if let Some(outputs) = run_param_fns(script_path, &fns, self.args, envs) {
             for (i, output) in outputs.into_iter().enumerate() {
                 let choices = output
                     .split('\n')
@@ -218,34 +238,50 @@ impl<'a, 'b> Matcher<'a, 'b> {
 
     pub(crate) fn to_arg_values_for_choice_fn(&self) -> Vec<ArgcValue> {
         let mut output: Vec<ArgcValue> = self.to_arg_values_base();
-        if !self.dashdash.is_empty() {
+        if !self.dashes.is_empty() {
             output.push(ArgcValue::Multiple(
-                "_dashdash".into(),
-                self.dashdash.iter().map(|v| v.to_string()).collect(),
+                "_dashes".into(),
+                self.dashes.iter().map(|v| v.to_string()).collect(),
             ));
         }
         output
     }
 
-    pub(crate) fn compgen(&self) -> Vec<(String, String)> {
+    pub(crate) fn compgen(&self) -> Vec<CompItem> {
         let level = self.cmds.len() - 1;
         let mut last_cmd = self.cmds[level].1;
         let mut output = match &self.arg_comp {
-            ArgComp::FlagOrOption => self.comp_flag_options(),
+            ArgComp::FlagOrOption => {
+                let mut output = self.comp_flag_options();
+                if let Some((value, param)) = self
+                    .args
+                    .last()
+                    .and_then(|value| last_cmd.find_flag_option(value).map(|param| (value, param)))
+                {
+                    let describe = param.describe_head();
+                    let kind = if param.is_flag() {
+                        CompKind::Flag
+                    } else {
+                        CompKind::Option
+                    };
+                    output.push((value.clone(), describe.into(), kind));
+                }
+                output
+            }
             ArgComp::FlagOrOptionCombine(value) => {
-                let mut output: Vec<(String, String)> = self
+                let mut output: Vec<CompItem> = self
                     .comp_flag_options()
                     .iter()
-                    .filter_map(|(x, y)| {
+                    .filter_map(|(x, y, z)| {
                         if x.len() == 2 {
-                            Some((format!("{value}{}", &x[1..]), y.to_string()))
+                            Some((format!("{value}{}", &x[1..]), y.to_string(), *z))
                         } else {
                             None
                         }
                     })
                     .collect();
                 if output.len() == 1 {
-                    output.insert(0, (value.to_string(), String::new()));
+                    output.insert(0, (value.to_string(), String::new(), CompKind::Flag));
                 }
                 output
             }
@@ -273,25 +309,20 @@ impl<'a, 'b> Matcher<'a, 'b> {
                 if self.positional_args.len() == 2 && self.positional_args[0] == "help" {
                     return comp_subcomands(last_cmd);
                 }
-                let mut output = vec![];
-                if last_cmd.positional_params.is_empty() {
-                    output.extend(self.comp_flag_options());
-                }
                 let values = self.match_positionals();
-                output.extend(comp_subcommands_positional(
-                    last_cmd,
-                    &values,
-                    self.positional_args.len() < 2,
-                ));
-                output
+                comp_subcommands_positional(last_cmd, &values, self.positional_args.len() < 2)
             }
         };
         if output.is_empty() && !self.arg_comp.is_flag_or_option() {
-            output.push(("__argc_value:file".into(), String::new()));
+            output.push(("__argc_value=file".into(), String::new(), CompKind::Value));
         }
         output
     }
 
+    pub(crate) fn is_last_arg_option_assign(&self) -> bool {
+        self.is_last_arg_option_assign
+    }
+
     fn to_arg_values_base(&self) -> Vec<ArgcValue> {
         let mut output = vec![];
         let cmds_len = self.cmds.len();
@@ -519,13 +550,13 @@ impl<'a, 'b> Matcher<'a, 'b> {
         while param_index < params_len && arg_index < args_len {
             let param = &cmd.positional_params[param_index];
             if param.multiple {
-                let dashdash_idx = self.dashdash.first().cloned().unwrap_or_default();
+                let dash_idx = self.dashes.first().cloned().unwrap_or_default();
                 let takes = if param_index == 0
-                    && dashdash_idx > 0
+                    && dash_idx > 0
                     && params_len == 2
                     && cmd.positional_params[1].multiple
                 {
-                    dashdash_idx
+                    dash_idx
                 } else {
                     (args_len - arg_index).saturating_sub(params_len - param_index) + 1
                 };
@@ -675,7 +706,7 @@ impl<'a, 'b> Matcher<'a, 'b> {
         (cmd, cmd_paths)
     }
 
-    fn comp_flag_options(&self) -> Vec<(String, String)> {
+    fn comp_flag_options(&self) -> Vec<CompItem> {
         let mut output = vec![];
         let level = self.cmds.len() - 1;
         let cmd = self.cmds[level].1;
@@ -687,8 +718,13 @@ impl<'a, 'b> Matcher<'a, 'b> {
             let exist = args.contains(param.name.as_str());
             if !exist || param.multiple {
                 let describe = param.describe_head();
+                let kind = if param.is_flag() {
+                    CompKind::Flag
+                } else {
+                    CompKind::Option
+                };
                 for v in param.list_names() {
-                    output.push((v, describe.to_string()))
+                    output.push((v, describe.to_string(), kind))
                 }
             }
         }
@@ -696,18 +732,20 @@ impl<'a, 'b> Matcher<'a, 'b> {
     }
 }
 
+pub(crate) type CompItem = (String, String, CompKind);
+
 fn add_positional_arg<'a>(
     positional_args: &mut Vec<&'a str>,
     arg: &'a str,
-    terminated: &mut bool,
+    is_rest_args_positional: &mut bool,
     cmd: &Command,
 ) {
     positional_args.push(arg);
-    if !*terminated
+    if !*is_rest_args_positional
         && cmd.positional_params.last().map(|v| v.terminated) == Some(true)
         && positional_args.len() >= cmd.positional_params.len() - 1
     {
-        *terminated = true;
+        *is_rest_args_positional = true;
     }
 }
 
@@ -790,7 +828,7 @@ fn comp_subcommands_positional(
     cmd: &Command,
     values: &[Vec<&str>],
     with_subcmd: bool,
-) -> Vec<(String, String)> {
+) -> Vec<CompItem> {
     let mut output = vec![];
     if with_subcmd {
         output.extend(comp_subcomands(cmd))
@@ -802,18 +840,18 @@ fn comp_subcommands_positional(
     output
 }
 
-fn comp_subcomands(cmd: &Command) -> Vec<(String, String)> {
+fn comp_subcomands(cmd: &Command) -> Vec<CompItem> {
     let mut output = vec![];
     for subcmd in cmd.subcommands.iter() {
         let describe = subcmd.describe_head();
         for v in subcmd.list_names() {
-            output.push((v, describe.to_string()))
+            output.push((v, describe.to_string(), CompKind::Command))
         }
     }
     output
 }
 
-fn comp_flag_option(param: &FlagOptionParam, index: usize) -> Vec<(String, String)> {
+fn comp_flag_option(param: &FlagOptionParam, index: usize) -> Vec<CompItem> {
     let value_name = param
         .arg_value_names
         .get(index)
@@ -828,7 +866,7 @@ fn comp_flag_option(param: &FlagOptionParam, index: usize) -> Vec<(String, Strin
     )
 }
 
-fn comp_positional(param: &PositionalParam) -> Vec<(String, String)> {
+fn comp_positional(param: &PositionalParam) -> Vec<CompItem> {
     comp_param(
         param.describe_head(),
         &param.arg_value_name,
@@ -844,7 +882,7 @@ fn comp_param(
     choices: &Option<Vec<String>>,
     choices_fn: &Option<(String, bool)>,
     multi_char: &Option<char>,
-) -> Vec<(String, String)> {
+) -> Vec<CompItem> {
     let choices: Option<Either<Vec<String>, String>> = if let Some(choices_fn) = choices_fn {
         Some(Either::Right(choices_fn.0.to_string()))
     } else {
@@ -856,16 +894,27 @@ fn comp_param(
         match choices {
             Either::Left(choices) => choices
                 .iter()
-                .map(|v| (v.to_string(), String::new()))
+                .map(|v| (v.to_string(), String::new(), CompKind::Value))
                 .collect(),
-            Either::Right(choices_fn) => vec![(format!("__argc_fn:{}", choices_fn), String::new())],
+            Either::Right(choices_fn) => vec![(
+                format!("__argc_fn={}", choices_fn),
+                String::new(),
+                CompKind::Value,
+            )],
         }
     } else {
-        let value = format!("__argc_value:{}", value_name);
-        vec![(value, describe.into())]
+        let value = format!("__argc_value={}", value_name);
+        vec![(value, describe.into(), CompKind::Value)]
     };
     if let Some(ch) = multi_char {
-        output.insert(0, (format!("__argc_multi:{}", ch), String::new()));
+        output.insert(
+            0,
+            (
+                format!("__argc_multi={}", ch),
+                String::new(),
+                CompKind::Value,
+            ),
+        );
     }
     output
 }
