@@ -14,7 +14,9 @@ pub(crate) struct ParamData {
     pub(crate) choices: Option<Vec<String>>,
     pub(crate) choices_fn: Option<(String, bool)>,
     pub(crate) multiple: bool,
+    pub(crate) multi_char: Option<char>,
     pub(crate) required: bool,
+    pub(crate) terminated: bool,
     pub(crate) default: Option<String>,
     pub(crate) default_fn: Option<String>,
 }
@@ -26,7 +28,9 @@ impl ParamData {
             choices: None,
             choices_fn: None,
             multiple: false,
+            multi_char: None,
             required: false,
+            terminated: false,
             default: None,
             default_fn: None,
         }
@@ -43,7 +47,9 @@ pub(crate) struct FlagOptionParam {
     pub(crate) choices: Option<Vec<String>>,
     pub(crate) choices_fn: Option<(String, bool)>,
     pub(crate) multiple: bool,
+    pub(crate) multi_char: Option<char>,
     pub(crate) required: bool,
+    pub(crate) terminated: bool,
     pub(crate) default: Option<String>,
     pub(crate) default_fn: Option<String>,
     pub(crate) value_names: Vec<String>,
@@ -81,7 +87,9 @@ impl FlagOptionParam {
             choices: arg.choices,
             choices_fn: arg.choices_fn,
             multiple: arg.multiple,
+            multi_char: arg.multi_char,
             required: arg.required,
+            terminated: arg.terminated,
             default: arg.default,
             default_fn: arg.default_fn,
             value_names,
@@ -114,7 +122,9 @@ impl FlagOptionParam {
                 &self.choices,
                 &self.choices_fn,
                 self.multiple,
+                self.multi_char,
                 self.required,
+                self.terminated,
                 &self.default,
                 &self.default_fn,
             );
@@ -128,7 +138,9 @@ impl FlagOptionParam {
                 &self.choices,
                 &self.choices_fn,
                 self.multiple,
+                self.multi_char,
                 self.required,
+                self.terminated,
                 &self.default,
                 &self.default_fn,
             );
@@ -242,10 +254,21 @@ impl FlagOptionParam {
                 return None;
             }
             if self.multiple {
-                let values: Vec<String> = values
+                let mut values: Vec<String> = values
                     .iter()
                     .flat_map(|v| v.iter().map(|v| v.to_string()))
                     .collect();
+                if let Some(c) = self.multi_char {
+                    values = values
+                        .into_iter()
+                        .flat_map(|v| {
+                            v.split(c)
+                                .filter(|v| !v.is_empty())
+                                .map(|v| v.to_string())
+                                .collect::<Vec<String>>()
+                        })
+                        .collect()
+                }
                 Some(ArgcValue::Multiple(name, values))
             } else if self.values_size() > 1 {
                 Some(ArgcValue::Multiple(
@@ -300,7 +323,9 @@ pub(crate) struct PositionalParam {
     pub(crate) choices: Option<Vec<String>>,
     pub(crate) choices_fn: Option<(String, bool)>,
     pub(crate) multiple: bool,
+    pub(crate) multi_char: Option<char>,
     pub(crate) required: bool,
+    pub(crate) terminated: bool,
     pub(crate) default: Option<String>,
     pub(crate) default_fn: Option<String>,
     pub(crate) value_name: Option<String>,
@@ -316,7 +341,9 @@ impl PositionalParam {
             choices: arg.choices,
             choices_fn: arg.choices_fn,
             multiple: arg.multiple,
+            multi_char: arg.multi_char,
             required: arg.required,
+            terminated: arg.terminated,
             default: arg.default,
             default_fn: arg.default_fn,
             value_name: value_name.map(|v| v.to_string()),
@@ -339,7 +366,9 @@ impl PositionalParam {
             &self.choices,
             &self.choices_fn,
             self.multiple,
+            self.multi_char,
             self.required,
+            self.terminated,
             &self.default,
             &self.default_fn,
         );
@@ -379,7 +408,18 @@ impl PositionalParam {
             return None;
         }
         if self.multiple {
-            let values: Vec<String> = values.iter().map(|v| v.to_string()).collect();
+            let mut values: Vec<String> = values.iter().map(|v| v.to_string()).collect();
+            if let Some(c) = self.multi_char {
+                values = values
+                    .into_iter()
+                    .flat_map(|v| {
+                        v.split(c)
+                            .filter(|v| !v.is_empty())
+                            .map(|v| v.to_string())
+                            .collect::<Vec<String>>()
+                    })
+                    .collect()
+            }
             Some(ArgcValue::PositionalMultiple(name, values))
         } else {
             Some(ArgcValue::PositionalSingle(name, must_get_first(values)))
@@ -394,18 +434,24 @@ impl PositionalParam {
     }
 }
 
+#[allow(clippy::too_many_arguments)]
 fn render_name(
     name: &str,
     choices: &Option<Vec<String>>,
     choices_fn: &Option<(String, bool)>,
     multiple: bool,
+    multi_char: Option<char>,
     required: bool,
+    terminated: bool,
     default: &Option<String>,
     default_fn: &Option<String>,
 ) -> String {
     let mut name = name.to_string();
     if let Some(choices) = choices {
-        if let Some(ch) = get_modifer(required, multiple) {
+        if let Some(ch) = get_modifer(required, multiple, terminated) {
+            name.push(ch)
+        }
+        if let Some(ch) = multi_char {
             name.push(ch)
         }
         let mut prefix = String::new();
@@ -425,7 +471,10 @@ fn render_name(
         let choices_value = format!("[{}{}]", prefix, values.join("|"));
         name.push_str(&choices_value);
     } else if let Some((choices_fn, validate)) = choices_fn {
-        if let Some(ch) = get_modifer(required, multiple) {
+        if let Some(ch) = get_modifer(required, multiple, terminated) {
+            name.push(ch)
+        }
+        if let Some(ch) = multi_char {
             name.push(ch)
         }
         let validate_sign = if *validate { "" } else { "?" };
@@ -439,13 +488,21 @@ fn render_name(
         let _ = write!(name, "={}", value);
     } else if let Some(default_fn) = default_fn {
         let _ = write!(name, "=`{}`", default_fn);
-    } else if let Some(ch) = get_modifer(required, multiple) {
-        name.push(ch)
+    } else {
+        if let Some(ch) = get_modifer(required, multiple, terminated) {
+            name.push(ch)
+        }
+        if let Some(ch) = multi_char {
+            name.push(ch)
+        }
     }
     name
 }
 
-fn get_modifer(required: bool, multiple: bool) -> Option<char> {
+fn get_modifer(required: bool, multiple: bool, terminated: bool) -> Option<char> {
+    if multiple && terminated {
+        return Some('~');
+    }
     match (required, multiple) {
         (true, true) => Some('+'),
         (true, false) => Some('!'),
