@@ -7,6 +7,8 @@ use std::{
 };
 use which::which;
 
+pub const INTERNAL_MODE: &str = "___internal___";
+
 /// Transform into upper case string with an underscore between words. `foo-bar` => `FOO-BAR`
 pub fn to_cobol_case(value: &str) -> String {
     Converter::new()
@@ -95,6 +97,7 @@ pub fn run_param_fns(
                 process::Command::new(shell)
                     .args(shell_extra_args)
                     .arg(&script_file)
+                    .arg(INTERNAL_MODE)
                     .arg(&param_fn)
                     .args(args)
                     .envs(envs)
@@ -122,13 +125,18 @@ pub fn termwidth() -> Option<usize> {
     env::var("TERM_WIDTH").ok()?.parse().ok()
 }
 
+pub fn is_windows_path(value: &str) -> bool {
+    let value = value.to_ascii_lowercase();
+    ('a'..='z').any(|v| value.starts_with(&format!("{v}:")))
+}
+
 pub fn get_current_dir() -> Option<String> {
     env::current_dir()
         .ok()
         .map(|v| v.to_string_lossy().to_string())
 }
 
-fn path_env_with_exe() -> String {
+pub fn path_env_with_exe() -> String {
     let mut path_env = std::env::var("PATH").ok().unwrap_or_default();
     if let Some(exe_dir) = std::env::current_exe()
         .ok()
