@@ -106,6 +106,7 @@ cmd::subb() { :; }
     snapshot_compgen!(
         SCRIPT,
         vec![
+            vec!["prog", ""],
             vec!["prog", "cmd"],
             vec!["prog", "cmd", ""],
             vec!["prog", "cmd", "s"],
@@ -331,6 +332,15 @@ cmdb() { :; }
     );
 }
 
+#[test]
+fn just_match() {
+    let script = r###"
+# @option --oa
+# @option --oa-file
+"###;
+    snapshot_compgen!(script, vec![vec!["prog", "--oa"]]);
+}
+
 #[test]
 fn no_flags_options() {
     let script = r###"
@@ -352,6 +362,34 @@ arg() { :; }
     );
 }
 
+#[test]
+fn no_flags_options2() {
+    const SCRIPT: &str = r###"
+# @option --oa
+# @option --ob
+
+# @cmd
+cmda() { :; }
+# @cmd
+cmdb() { :; }
+"###;
+
+    snapshot_compgen!(SCRIPT, vec![vec!["prog", ""],]);
+}
+
+#[test]
+fn one_subcmd_with_options() {
+    const SCRIPT: &str = r###"
+# @option --oa
+# @option --ob
+
+# @cmd
+cmda() { :; }
+"###;
+
+    snapshot_compgen!(SCRIPT, vec![vec!["prog", ""],]);
+}
+
 #[test]
 fn no_space() {
     let script = r###"
@@ -386,8 +424,8 @@ fn multi_parts() {
     let script = r###"
 # @option --oa*[`_choice_fn`]
 _choice_fn() {
-    echo __argc_prefix:A/
-    echo __argc_matcher:''
+    echo __argc_prefix=A/
+    echo __argc_filter=''
 	echo B
 	echo -e "B/\0"
 }
@@ -401,8 +439,8 @@ fn multi_parts2() {
     let script = r###"
 # @option --oa*[`_choice_fn`]
 _choice_fn() {
-    echo __argc_prefix:A/B/
-    echo __argc_matcher:''
+    echo __argc_prefix=A/B/
+    echo __argc_filter=''
 	echo
 	echo C
 	echo D
@@ -413,27 +451,13 @@ _choice_fn() {
 }
 
 #[test]
-fn suffix() {
-    let script = r###"
-# @option --oa*[`_choice_fn`]
-_choice_fn() {
-    echo __argc_suffix:=
-    echo -e "A\0"
-    echo -e "B\0"
-}
-"###;
-
-    snapshot_compgen_shells!(script, vec!["prog", "--oa", ""]);
-}
-
-#[test]
-fn option_equal() {
+fn assing_option_value() {
     let script = r###"
 # @option --oa[`_choice_fn`]
 # @arg val[`_choice_fn`]
 _choice_fn() {
-    echo __argc_matcher:
-    ( set -o posix ; set ) | grep 'argc_\|ARGC_MATCHER'
+    echo __argc_filter=
+    ( set -o posix ; set ) | grep 'argc_\|ARGC_' | grep -v 'ARGC_PWD\|ARGC_PATH_SEP\|ARGC_OS'
 }
 "###;
 
@@ -449,7 +473,8 @@ fn arg_terminated() {
 # @arg cmd
 # @arg args~[`_choice_fn`]
 _choice_fn() {
-    echo __argc_matcher:
+    echo __argc_filter=
+    echo ${argc__positionals[@]}
     echo ok
 }
 "###;
@@ -459,6 +484,7 @@ _choice_fn() {
         vec![
             vec!["sudo", "cmd", ""],
             vec!["sudo", "cmd", "-"],
+            vec!["sudo", "cmd", "--"],
             vec!["sudo", "cmd", "--foo"],
             vec!["sudo", "cmd", "foo"],
         ]
@@ -471,7 +497,7 @@ fn option_terminated() {
 # @option --oa~[`_choice_fn`]
 # @option --ob
 _choice_fn() {
-    echo __argc_matcher:
+    echo __argc_filter=
     echo ok
 }
 "###;
@@ -488,6 +514,28 @@ _choice_fn() {
     );
 }
 
+#[test]
+fn last_arg_option_assign() {
+    let script = r###"
+# @option --oa~[`_choice_fn`]
+# @option --ob <file>
+# @arg args~[`_choice_fn`]
+_choice_fn() {
+    echo __argc_filter=
+    echo ok
+}
+"###;
+
+    snapshot_compgen!(
+        script,
+        vec![
+            vec!["prog", "--ob="],
+            vec!["prog", "--oa", "--ob="],
+            vec!["prog", "abc", "--ob="],
+        ]
+    );
+}
+
 #[test]
 fn fallback_comp_file() {
     let script = r###"
@@ -605,17 +653,195 @@ _choice_fn() {
 }
 
 #[test]
-fn generic_shell() {
+fn bash_shell() {
     let script = r###"
 # @option --oa[`_choice_fn`]
+# @option --ob[`_choice_fn2`]
+# @option --oc <file>
+# @flag   --fa
 _choice_fn() {
-    echo -e "__argc_prefix:'"
-    echo -e "__argc_matcher:"
-    echo -e "abc\t(desc 1)"
-    echo -e "def\0"
-    echo -e "ijk"
+	echo "abc:def:xyz"
+	echo "abc:def:tsr"
+	echo "abc:ijk:abc"
+	echo "abc:ijk:xyz"
+}
+_choice_fn2() {
+    echo __argc_prefix=/A/
+    echo __argc_filter=B
+    echo -e "B"
+    echo -e "B/\0"
 }
 "###;
 
-    snapshot_compgen!(script, vec![vec!["prog", "--oa", ""]], argc::Shell::Generic);
+    snapshot_compgen!(
+        script,
+        vec![
+            vec!["prog", "--oa", ""],
+            vec!["prog", "--oa", "abc:"],
+            vec!["prog", "--ob", "/A/B"],
+            vec!["prog", "--oc", "foo="],
+            vec!["prog", "-"]
+        ],
+        argc::Shell::Bash
+    );
+}
+
+#[test]
+fn kinds_of() {
+    let script = r###"
+# @arg val[`_choice_fn`]
+_choice_fn() {
+    echo -e "a1\tdesc a1"
+    echo -e "b1\0\tdesc b1"
+    echo -e "c1\t"
+    echo -e "d1\0"
+    echo -e "e1"
+    echo -e "f1\t/kind:valueOther\tdesc f1"
+}
+"###;
+
+    snapshot_compgen_shells!(script, vec!["prog", ""]);
+}
+
+#[test]
+fn filter_quote() {
+    let script = r###"
+# @arg args[`_choice_fn`]
+_choice_fn() {
+    echo "__argc_prefix=${ARGC_FILTER%%=*}="
+    echo "__argc_filter=${ARGC_FILTER#*=}"
+    echo foo
+    echo bar
+    :;
+}
+"###;
+
+    snapshot_compgen_shells!(script, vec!["prog", "v='"]);
+}
+
+#[test]
+fn redirect_symbols() {
+    let script = r###"
+# @option --oa
+# @arg text*
+"###;
+
+    snapshot_compgen_shells!(script, vec!["prog", ">", ""]);
+}
+
+#[test]
+fn color() {
+    let script = r###"
+# @arg val[`_fn_color`]
+_fn_color() {
+    echo -e "flag\0\t/kind:flag"
+    echo -e "option\0\t/kind:option"
+    echo -e "command\0\t/kind:command"
+    echo -e "dir\0\t/kind:dir"
+    echo -e "file\0\t/kind:file"
+    echo -e "fileExe\0\t/kind:fileExe"
+    echo -e "symlink\0\t/kind:symlink"
+    echo -e "valueOther\0\t/kind:valueOther"
+    echo -e "valueAnother\0\t/kind:valueAnother"
+    echo -e "valueEmphasis\0\t/kind:valueEmphasis"
+    echo -e "valueSubtle\0\t/kind:valueSubtle"
+    echo -e "value\0\t/kind:value"
+}
+"###;
+    snapshot_compgen_shells!(script, vec!["prog", ""]);
+}
+
+mod filedir {
+    #[cfg(windows)]
+    const TEST_SHELL: argc::Shell = argc::Shell::Powershell;
+    #[cfg(not(windows))]
+    const TEST_SHELL: argc::Shell = argc::Shell::Elvish;
+
+    const VALUE_NAME_SCRIPT: &str = r###"
+# @option --oa <file>
+# @option --ob <file:.zsh>
+# @option --oc <dir>
+"###;
+
+    #[cfg(not(windows))]
+    #[test]
+    fn value_name() {
+        snapshot_compgen!(
+            VALUE_NAME_SCRIPT,
+            vec![
+                vec!["prog", "--oa", "src/"],
+                vec!["prog", "--oa", "src/p"],
+                vec!["prog", "--oa", "./src/"],
+                vec!["prog", "--oa", "C"],
+                vec!["prog", "--oa", "./C"],
+                vec!["prog", "--ob", "src/bin/argc/completions/ar"],
+                vec!["prog", "--oc", "src/"],
+            ],
+            TEST_SHELL
+        );
+    }
+
+    #[cfg(windows)]
+    #[test]
+    fn value_name_win() {
+        snapshot_compgen!(
+            VALUE_NAME_SCRIPT,
+            vec![
+                vec!["prog", "--oa", "src\\"],
+                vec!["prog", "--oa", "src\\p"],
+                vec!["prog", "--oa", ".\\src\\"],
+                vec!["prog", "--oa", "C"],
+                vec!["prog", "--oa", ".\\C"],
+                vec!["prog", "--ob", "src\\bin\\argc\\completions\\ar"],
+                vec!["prog", "--oc", "src\\"],
+            ],
+            TEST_SHELL
+        );
+    }
+
+    const CD_SCRIPT: &str = r###"
+# @option --oa[`_choice_fn`]
+# @arg val[`_choice_fn2`]
+_choice_fn() {
+    echo "__argc_cd=src"
+    echo "__argc_value=file"
+}
+_choice_fn2() {
+    if [[ "$1" == *"="* ]]; then
+        echo __argc_prefix=${1%%=*}=
+        echo __argc_filter=${1#*=}
+        echo __argc_cd=src
+        echo __argc_value=file
+
+    fi
+}
+    "###;
+
+    #[cfg(not(windows))]
+    #[test]
+    fn cd() {
+        snapshot_compgen!(
+            CD_SCRIPT,
+            vec![
+                vec!["prog", "--oa", ""],
+                vec!["prog", "--oa="],
+                vec!["prog", "foo="]
+            ],
+            TEST_SHELL
+        );
+    }
+
+    #[cfg(windows)]
+    #[test]
+    fn cd_win() {
+        snapshot_compgen!(
+            CD_SCRIPT,
+            vec![
+                vec!["prog", "--oa", ""],
+                vec!["prog", "--oa="],
+                vec!["prog", "foo="]
+            ],
+            TEST_SHELL
+        );
+    }
 }
